%"Beyond Rank-1: Discovering Rich Community Structure in Multi-Aspect Graphs" 
function [ H, his ] = cLL1_AOadmm( Y, k, ops )
% AO-ADMM for constrained tensor block term  decomposition with least-squares loss
% input: Y, data; k, list of ranks;
%        ops.loss          = 'ls'          -- default
%                          = 'l1'
%                          = 'KL'
%        ops.constraint{d} = 'nonnegative' -- default
%                          = 'l1'      -- ops.l1{d}  = penalty
%                          = 'l1n'          l1 and nonnegative
%                          = 'l2'      -- ops.l2{d}  = penalty
%                          = 'l2n'          l2 and nonnegative
%                          = 'l2-bound'-- row norm less equal to 1
%                          = 'l2-boundn'    l2bound and nonnegative
%                          = 'bias'    -- let the d-th column being 1
%                                               + nonnegativity
%        ops.init          = I_init
%        ops.maxitr        = max number of iterations
%        ops.maxtime       = max time
%        ops.mu            = BSUM Tikhonov regularization parameter
%       for ls loss and nonnegative constraint, you can simply set ops=[]
% output: H, matrix cell of the factors
%        his.time          = time elapsed per iteration
%        his.err           = objective value per iteration
%        his.admm          = number of admm iterates per iteration
global timeSpent iter;
tic

dim = length(size(Y)); nn = size(Y);
H = cell( dim, 1 ); U = cell( dim, 1 );
L=length(k);LR=sum(k);
for d = 1:dim
    if d==dim
        H{d} =  rand( nn(d), L);
        U{d} = zeros( nn(d), L);
    else
        H{d} =  rand( nn(d), LR);
        U{d} =  zeros( nn(d), LR);
    end
    H{d} = H{d} / diag( sqrt( sum( H{d}.^2 ) ) );
   
end
if ~isfield(ops, 'loss'),       ops.loss = 'ls';                end
if ~isfield(ops, 'constraint'), ops.constraint = cell(dim,1);
    for d=1:dim, ops.constraint{d} = 'nonnegative'; end;        end
if  isfield(ops, 'Hinit'),       H = ops.Hinit;                   end
if ~isfield(ops, 'maxitr' ),    ops.maxitr = 5;               end
if ~isfield(ops, 'maxtime'),    ops.maxtime = 6000;              end
if ~isfield(ops, 'tol'),        ops.tol = 1e-7;                 end
GG = cell( dim, 1 );
for d = 1:dim
    GG{d} = H{d}'*H{d};
end
ops.mu = 0;
timeSpent = zeros(ops.maxitr, size(Y, 3));

if ~strcmp( ops.loss, 'ls' ), ops.maxitr = round(ops.maxitr/2); end
his.time    = zeros(1,ops.maxitr);
his.err     = zeros(1,ops.maxitr);
his.fit     = zeros(1,ops.maxitr);
his.admm    = zeros(dim,ops.maxitr);
his.LossCalcTime=zeros(1,ops.maxitr);
%%  tensor case, using Kolda's tensor_toolbox, sptensor

    for itr = 1:ops.maxitr
        iter = itr;
        for d = 1:dim
            [ H, U, GG, i ] =  cLL1_fastADMM( Y, H, U, d, GG,k, ops);
            his.admm(d,itr) = i;
        end
        his.time(itr) = toc;
        [fit,err ,ft] = Loss( Y, H, ops, k );
        his.err(itr)=err;
        his.fit(itr)=fit;
        his.LossCalcTime(itr)=ft;
        ops.mu = 1e-7 + .1*his.err(itr);
% 
%         if toc > ops.maxtime || ( itr>1 && ...
%                     abs( his.err(itr) - his.err(itr-1) ) < ops.tol ) ...
%                     ||(itr>1 && his.fit(itr) < his.fit(itr-1))
%             his.err(itr+1:end) = [];
%             his.fit(itr+1:end) = [];
%             his.time(itr+1:end) = [];
%             his.admm(:,itr+1:end) = [];
%       
%             break; 
%         end
        fprintf('Iteration %d ..... \n',itr); % uncomment line below and line 99 for fitness, it will take time to compute the fitness
        %fprintf('Iteration %d ..... fit:%f \n',itr,abs(fit));
    end
    
 
end
function [fit,err ,ft]= Loss( Y, H, ops,k )
    st=tic;
    switch ops.loss
        case 'ls'
                Y=sptensor(Y);
                %[fit,err]=calcFitness(H{1},H{2},H{3},size(Y),length(k),Y,k);
               fit=1;err=0;
        case 'KL'
                Yt = max( double( ktensor(H) ), eps ); Err = Yt;
                KL = Y .* log( Y./Yt ) - Y + Yt;
                Err( Y~=0 ) = KL( Y~=0 );
                err = sum( Err(:) );
                fit=1;
        case 'l1'
            Err = abs( Y - double(ktensor(H)) );
            err = sum( Err(:) );fit=1;

    end
    ft=toc(st)-st;
end
function [fit,err]=calcFitness(A,B,C,dims,L,X,R)
       X=sptensor(X);
       normX=norm(X);
       P=sptensor(dims);
       stIdx=1;
       for r =1:L
           endIdx=stIdx+R(r)-1;
%            AB=A(:,stIdx:endIdx)*B(:,stIdx:endIdx)';
%            c=C(:,r);       
%            Y=rLL1_kronmult(AB, c);  %
           Y=kron(A(:,stIdx:endIdx)*B(:,stIdx:endIdx)',C(:,r));
           Y=reshape(Y,dims);
           P=P+sptensor(Y);
           stIdx=endIdx+1;
       end
    normresidual = sqrt( normX^2 + norm(P)^2 - 2 * innerprod(X,P) );
    fit = 1 - (normresidual / normX); %fraction explained by model
    err=(normresidual / normX);
end
