%"Beyond Rank-1: Discovering Rich Community Structure in Multi-Aspect Graphs" 

function [ H, U, GG, itr ] = cLL1_fastADMM( Y, H, U, d, GG,R, ops)
%fprintf('Processing dim : %d \n',d);
% ADMM iterates to solve
%       minimize (1/2)*|| Y - \sum_i=1_R((A_i.B_i')*c_i) ||^2 + r(H{d})
% for sparse tensor factorization
%   Y is the 3-mode tensor
%   H is a cell of the matrix factors
%   d is the index of the matrix factor to be updated

[ ~, k ] = size(H{d});
G= H{d}'*H{d};
rho = min(1e-3,trace(G)/k);
[G] = topdm(G) ;
% L= chol( G ,'lower');
% L = chol( G + (rho+ops.mu)*eye(k), 'lower' );
if d==1
    F=updateFactorA(sptensor(Y),H,R,d);
elseif d==2
    F=updateFactorB(sptensor(Y),H,R,d);
else
    F=updateFactorC(sptensor(Y),H,R,d);
end

tol = 1e-4;
Hd = H{d}; Ud = U{d};
if ~isfield(ops, 'maxitrAO')
    maxitr = 5;
else
    maxitr=ops.maxitrAO;
end
for itr = 1:maxitr
    H0 = Hd;
    %% Fix it 
%     Ht=L'\ ( L\ ( F + rho*(Hd+Ud) + ops.mu*H{d} )');
      Ht   = (F+ops.mu*H{d})';%( F + rho*(Hd+Ud) + ops.mu*H{d} )'; %L'\ ( L\ ( F + rho*(Hd+Ud) + ops.mu*H{d} )'); 
    %% 
    Hd = proxr( Ht'-Ud, ops, d, rho,Hd);
    Ud = Ud + Hd - Ht';
    
    r = Hd - Ht';
    s = Hd - H0;
    if norm(r(:)) < tol*norm(Hd(:)) && norm(s(:)) < tol*norm(Ud(:))
        break
    end
end
H{d} = Hd;
U{d} = Ud;
GG{d} = Hd'*Hd;
end

function A=updateFactorA(X,H,R,d)
      % unfold the tensors in all three modes
      T= transpose(double(sptenmat(X,d)));
      T = sparse(T); % T_JKxI
    
      % get factor matrix
        B=sparse(H{2});C=sparse(H{3});
      % start update
       L=length(R);
       stIdx=1;
       
       for r=1:L
           endIdx=stIdx+R(r)-1;
           U(:,stIdx:endIdx)=sparse(kron(B(:,stIdx:endIdx),C(:,r)));
           stIdx=endIdx+1;
       end
        A=transpose(pinv(full(U))*T);% update A
      %  A=transpose(pinv_rLL1(U)*T);% update A
end
function B=updateFactorB(X,H,R,d)
      % unfold the tensors in all three modes
        T = transpose(double(sptenmat(X,d))); % T_JKxI
      % get factor matrix
        A=H{1};C=H{3};
      % start update
       L=length(R);
       % update B only, fix A and C
       stIdx=1;
       for r=1:L
           endIdx=stIdx+R(r)-1;
           V(:,stIdx:endIdx)=kron(C(:,r),A(:,stIdx:endIdx));
           stIdx=endIdx+1;
       end
      % B = transpose(pinv_rLL1(V)*T);% update B
       B=transpose(pinv(full(V))*T);% update A
end
function C=updateFactorC(X,H,R,d)
      % unfold the tensors in all three modes
        T = transpose(double(sptenmat(X,d))); % T_JKxI
      % get factor matrix
        A=H{1};B=H{2};
      % start update
       L=length(R);
      % update C only, fix B and A
       stIdx=1;
       for r=1:L
           endIdx=stIdx+R(r)-1;
           W(:,r)=  mtimes(krb(A(:,stIdx:endIdx),B(:,stIdx:endIdx)),ones(R(r),1));     
           stIdx=endIdx+1;
       end
       %C=transpose(pinv_rLL1(W)*T); % update C
        C=transpose(pinv(full(W))*T);% update A
end
function H = proxr( Hb, ops, d, rho ,Hd)
    switch ops.constraint{d}
        case 'nonnegative'
            H = max( 0, Hb );
       case 'sparsity'
            H = max( 0, Hb )+ops.la*abs(Hb);
       case 'fln'
            H = fusedLASSO(Hb, ops, rho, Hd);
        case 'l1'
            H = sign( Hb ) .* max( 0, abs(Hb) - (ops.l1/rho) );
        case 'l1n'
            H = max( 0, Hb - ops.l1/rho );
        case 'l2'
            H = ( rho/(ops.l2{d}+rho) ) * Hb;
        case 'l2n'
            H = ( rho/(ops.l2{d}+rho) ) * max(0,Hb);
        case 'l2-bound'
           nn = sqrt( sum( Hb.^2 ) );
            H = Hb * diag( 1./ max(1,nn) );
        case 'l2-boundn'
            H = max( 10e-8, Hb );
           nn = sqrt( sum( H.^2 ) );
            H = H * diag( 1./ max(1,nn) );
        case 'l0'
            T = sort(Hb,2,'descend');
            t = T(:,4); T = repmat(t,1,size(T,2));
            H = Hb .* ( Hb >= T );
    end
end
function [sigma] = topdm(sig) 
% FUNCTION THAT TRANSFORMS A NON POSITIVE DEFINITE SYMMETRIC
% MATRIX -E.G. A COVARIANCE MATRIX- TO A 
% POSITIVE DEFINITE SYMMETRIC MATRIX. 
EPS = 10^-6;      %SET THE VALUE TO PLACE INSTEAD OF ZERO OR NEGATIVE  
                  %EIGENVALUES  
ZERO = 10^-10;    %SET THE VALUE TO LOOK FOR 
sigma = sig; 
[~, err] = cholcov(sigma, 0); %ERR<>0 MEANS SOME EIGENVALUES <=0 
                              %So... SIGMA IS NOT POSITIVE DEFINITE 
                              %So... SIGMA IS NOT INVERTIBLE 
if (err ~= 0) 
    [v ,d] = eig(sigma); %CALCULATE EIGENVECTOR AND EIGENVALUES 
    d=diag(d);          %GET A VECTOR OF EIGENVALUES 
    d(d<=ZERO)=EPS; %FIND ALL EIGENVALUES<=ZERO AND CHANGE THEM FOR EPS 
     
    d=diag(d);      %CONVERT VECTOR d INTO A MATRIX 
    sigma = v*d*v'; %RECOMPOSE SIGMA MATRIX USING EIGENDECOMPOSITION 
                    %WHY? SIGMA IS SIMETRIC AND V IS ORTHONORMAL 
end 
end


function H = fusedLASSO(Hb, ops, rho, A)
   % fixedStep = isfield(ops,'fixedStep'); %Do fixed step size instead of line search (speed)
    H = A;
    tol = 1e-2;
    [T, ~] = size(H);
    STEP = ops.startStep;
    BETA = ops.stepMult;
    for itr = 1:20
        A0 = H;
        for t = 1:T
            dA = (sign(H(t,:)))*ops.la + rho*(H(t,:)-Hb(t,:));
%             if t > 1
%                 dA = dA + ops.lb*sign(H(t,:)-H(t-1,:));
%             end
%             if t < T
%                 dA = dA - ops.lb*sign(H(t+1,:) - H(t,:));
%             end
%             s = STEP;
            Atemp = H;
            Atemp(t,:) = max(0, H(t,:) - dA*STEP);
%             i = 0;
%             if fixedStep, Atemp(t,:) = max(0, H(t,:) - dA*ops.fixedStep);
%             else
%                 baseline = sum(sum(abs(H(t,:))))*ops.la + sum(sum(abs(diff(H(max(t-1,1):min(t+1,T),:)))))*ops.lb + rho/2*norm(H(t,:)-Hb(t,:),'fro')^2;
%                 while s > ops.minStep
%                     i = i + 1;
%                     Atemp(t,:) = max(0,H(t,:) - dA*s);
%                     if sum(sum(abs(Atemp(t,:))))*ops.la + sum(sum(abs(diff(Atemp(max(t-1,1):min(t+1,T),:)))))*ops.lb + rho/2*norm(Atemp(t,:)-Hb(t,:),'fro')^2 < baseline
%                         break
%                     end
%                     Atemp = H;
%                     s = s*BETA;
%                 end
%             end
            H = Atemp;
        end
        r = A0 - H;
        if norm(r(:)) < tol*norm(H(:))
            break
        end
    end
end