%"Beyond Rank-1: Discovering Rich Community Structure in Multi-Aspect Graphs",  

function [Fac,timeSpent,his,model,orderedMatrix]=richcomm(X,R,Y,ops) 
 % Input : X - 3 mode tensor , L- known list of rank of tensor X, ops- supported parameters for rLL1
 % Output: Fac: Factor matrices[A,B and C]  and  timeSpent - Total CPU time

    if ~isfield(ops, 'la'), ops.la = .001; end %reg param on sparseness
    if ~isfield(ops, 'lb'), ops.lb = ops.la; end
    if ~isfield(ops, 'minStep'), ops.minStep = 10^-10; end
    if ~isfield(ops, 'startStep'), ops.startStep = 0.01; end
    if ~isfield(ops, 'stepMult'), ops.stepMult = 0.1; end
    sz=size(X);
    I = sz(1);J=sz(2);K=sz(3);
     
    if ~isfield(ops,'Hinit') 
        L=length(R);LR=sum(R);
        Hinit{1} = rand( I, LR );
        Hinit{2} = rand( J, LR );
        Hinit{3} = rand( K, L);
        for d = 1:3
            Hinit{d} = Hinit{d} / diag( sqrt( sum( Hinit{d}.^2 ) ) );
        end
        ops.Hinit = Hinit;
    end
    %% cLL1_AOadmm
    doFit = tic;
    [ Fac, his ] = cLL1_AOadmm( X, R, ops );
   
    %% normalizing the factor matrices.
    Fac=getresults(Fac{1},Fac{2},Fac{3},R);
    
    %% evaluation and get communities in  file , one line per community
    Dr=0;
    for r=1:size(L,2)
        Dr=Dr+Fac{r}{1}*Fac{r}{2}';
    end
 
     OrigMatrix=Dr;%-DiagVal;
     orderedMatrix=zeros(size(OrigMatrix));
     noOfComm=max(Y(:,2));
    clear DiagVal A B C D orderla;
    model = struct('Structure', {},  'Nodes', {},   'benefit', {}, 'benefit_notEnc', {},'Comm', {},'extranodes',{});
    start=1;
    for i =1:noOfComm
         f=find(Y(:,2)==i);
         curind=Y(f,1)';endidx=start+size(curind,2)-1;
         [StructureName,costGain,costGain_notEnc,Asmall,extranodes]= richcomm_EncodeCommunity( OrigMatrix, curind );
         orderedMatrix(start:endidx,start:endidx)=Asmall;
         start=endidx+1;
%        fprintf('Community => %d is of structure => %s with cost => %f and with cost encoding=> %f \n',i,StructureName,costGain,costGain_notEnc) ;
         model(i) = struct('Structure', StructureName, 'Nodes', curind,  'benefit', abs(costGain), 'benefit_notEnc', abs(costGain_notEnc),'Comm', 1,'extranodes',extranodes);
    end

    model=richcomm_getStructures(Fac,model,R);
    model=getuniqueVals(model);% get unique model structures (optional)
     
 timeSpent = toc(doFit);

end
function U=getresults(A,B,C,R)
% A(A<10e-3)=0;B(B<10e-3)=0;C(C<10e-3)=0;

L=size(R,2);
U=cell(1,L);
stIdx=1;
for r=1:L
    endIdx=stIdx+R(r)-1;
    U{r}{1}=A(:,stIdx:endIdx);
    U{r}{2}=B(:,stIdx:endIdx);
    U{r}{3}=C(:,r);
    stIdx=endIdx+1;
end

end
function model=richcomm_getStructures(Fac,model,R)
        
        L=max(size(R));
         
         for i=1:L
            noOfComm=length(model);
            Dr=Fac{i}{1}*Fac{i}{2}';      
            threshold=mean(mean(Dr));
            Dr(Dr<threshold)=0;
            Dr(Dr>=threshold)=1;
            [S, indexComp] = graphconncomp(sparse(Dr),'WEAK', true);
            n=1;
            for j =1:S
           
                curind=find(indexComp(1,:)==j);
                if size(curind,2)~=1
                    [StructureName,costGain,costGain_notEnc,~,extranodes]= richcomm_EncodeCommunity( Dr, curind );
%                   fprintf('Community => %d is of structure => %s with cost => %f and with cost encoding=> %f \n',noOfComm+n,StructureName,abs(costGain),abs(costGain_notEnc)) ;
                    model(n+noOfComm) = struct('Structure', StructureName, 'Nodes', curind,  'benefit', abs(costGain), 'benefit_notEnc', abs(costGain_notEnc),'Comm', 0,'extranodes',extranodes);
                    n=n+1;
               end
            end
            
            [model] = getGraph(Dr,model);
            
         end
end
function [model] = getGraph(Dr,model)
       DrOrig=Dr;
       
       n = size(Dr,1);k=10;noOfComm=length(model);
       row=1;M=1:n;
       while k<n 
           D=sum(Dr,2);
           [Dsort,I]=sort(D);
           top_index = flipud(I(n-k+1:n));
           
           Dr(top_index, :) = 0;
           Dr(:, top_index) = 0;
           [S,indexComp]=graphconncomp(sparse(Dr), 'WEAK', true);
           for j =1:S
                curind=M(find(indexComp(1,:)==j));
                if max(size(curind))~=1
                    [StructureName,costGain,costGain_notEnc,~,extranodes]= richcomm_EncodeCommunity( DrOrig, curind );
%                     fprintf('Community => %d is of structure => %s with cost => %f and with cost encoding=> %f \n',noOfComm+row,StructureName,abs(costGain),abs(costGain_notEnc)) ;
                    model(row+noOfComm) = struct('Structure', StructureName, 'Nodes', curind,  'benefit', abs(costGain), 'benefit_notEnc', abs(costGain_notEnc),'Comm', 0,'extranodes',extranodes);
                    row=row+1;
                end
           end
           M=setdiff(I,top_index);
           Dr=Dr(M,M);
           n = size(Dr,1);
           if sum(sum(Dr))==0
               break
           end
       end
 
end
function [strName,costGain,costGain_notEnc,Asmall,extranodes] = richcomm_EncodeCommunity( B, curind )
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   find whether it is clique, near-clique, star, chain or bipartite-core %
%   minSize: smallest size of reported structures (number of nodes)       %                                 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
N_tot=size(B,1);extranodes=0;
Asmall = B(curind,curind);
maxval=max(max(Asmall));
if maxval~=1
    avgVal=median(mean(Asmall));
    Asmall(Asmall<avgVal)=0;
    Asmall(Asmall>=avgVal)=1;
end
n = max(size(curind));
m = nnz(Asmall);

%% First try to find one of the synthetic structures (vocab words):
%   clique, star, chain, bipartite core
[exact_found,strName,costGain,costGain_notEnc] = richcomm_ExactCommStructure( Asmall, curind, N_tot );

if ( exact_found == false )
    
    E(2) = m;   E(1) = n^2 - n - E(2);
    %% MDL cost of encoding given substructure as a full clique
     MDLcostFC = richcomm_Cost( 'fc', N_tot, n, E);
    %% MDL cost of encoding given substructure as a near clique
    MDLcostNC = richcomm_Cost( 'nc', N_tot, n, Asmall);
    deg = full(sum(Asmall));[ ~, idxMaxDeg ] = max(deg);
    if idxMaxDeg ~= 1 && idxMaxDeg ~= n
        satellitesIdx = [1 : (idxMaxDeg-1), (idxMaxDeg+1):n];
    elseif idxMaxDeg == 1
        satellitesIdx = 2:n;
    elseif idxMaxDeg == n
        satellitesIdx = 1:(n-1);
    end
    E(1) = 2* (n-1-nnz(Asmall(idxMaxDeg,:))) + nnz(Asmall(satellitesIdx, satellitesIdx));E(2) = n^2 - E(1);

    MDLcostST = richcomm_Cost( 'st', N_tot, n, E);
    
    [ MDLcostBC, MDLcostNB, set1, set2 ] = mdlCostAsBCorNB( Asmall, N_tot );
   
    MDLcosts = [ MDLcostFC, MDLcostNC, MDLcostST,MDLcostBC, MDLcostNB];
    
    if m == 2*n
       % [ MDLcostCH, chain ] = mdlCostAsChain( Asmall, N_tot );
        % MDLcosts = [ MDLcosts, MDLcostCH ];
            MDLcostCH = richcomm_Cost( 'ch', N_tot, n, zeros(n,n));
            costGain =  MDLcostCH;
            costGain_notEnc = richcomm_Cost( 'err', 0, 0, [nnz(Asmall) n^2-nnz(Asmall)]);
            MDLcosts = [ MDLcosts, MDLcostCH ];
        strName='Chain';
    end
 
    [ ~, idxMin ] = min(MDLcosts);
    
    cost_notEnc = richcomm_Cost( 'err', 0, 0, [nnz(Asmall) n^2-nnz(Asmall)]);
    
    if isinf(MDLcosts(idxMin)) || isinf(MDLcosts(2))
        costGain_notEnc = cost_notEnc - MDLcostNC;strName='Near Clique' ;
     else
        switch idxMin
            case 1
                costGain = MDLcostNC - MDLcostFC;costGain_notEnc = cost_notEnc - MDLcostFC;strName='Full Clique' ;extranodes=0;
            case 2
                costGain = MDLcostNC - MDLcostNC;costGain_notEnc = cost_notEnc - MDLcostNC;strName='Near Clique' ;extranodes=0;
            case 3
                costGain = MDLcostNC - MDLcostST;costGain_notEnc = cost_notEnc - MDLcostST;strName='Star' ;extranodes=curind(satellitesIdx);
            case 4
                costGain = MDLcostNC - MDLcostBC;costGain_notEnc = cost_notEnc - MDLcostBC;strName='Bipartite' ;
                if sum(set1)~=0 && sum(set2)==0
                    extranodes= curind(set1);
                elseif sum(set1)==0 && sum(set2)~=0
                    extranodes= curind(set2);
                elseif sum(set1)==0 && sum(set2)==0
                    extranodes=0;
                else
                    x=curind(set1);if size(x,1)>1;x=x';end
                    y=curind(set2);if size(y,1)>1;y=y';end
                    extranodes=[x,0,y]; 
                end
            case 5
                costGain = MDLcostNC - MDLcostNB;costGain_notEnc = cost_notEnc - MDLcostNB;strName='Near Bipartite' ;
                if sum(set1)~=0 && sum(set2)==0
                    extranodes= curind(set1);
                elseif sum(set1)==0 && sum(set2)~=0
                    extranodes= curind(set2);
                elseif sum(set1)==0 && sum(set2)==0
                    extranodes=0;
                else
                    x=curind(set1); if size(x,1)>1;x=x';end
                    y=curind(set2); if size(y,1)>1;y=y';end
                    extranodes=[x,0,y ]; 
                end
            case 6
                costGain = MDLcostNC - MDLcostCH;costGain_notEnc = cost_notEnc - MDLcostCH;strName='Chain' ;extranodes=0;
            otherwise
                error_message = 'error: impossible to get this error...\n';
                disp(error_message);
        end
    end
    
end

end

function [ MDLcostBC, MDLcostNB, set1, set2 ] = mdlCostAsBCorNB( Asmall, N_tot )
 
%% Constants and variables of FaBP
% heterophily factor
h = 0.02;
% prior belief for belonging in the pos/neg class
positive = 0.02;
negative = -0.02;

a = 4*h^2/(1-4*h^2);
c = 2*h/(1-4*h^2);

%% setting up the matrices and vectors involved in FaBP
n = size(Asmall, 2);
deg = full(sum(Asmall));
%D = diag(deg);
D = spdiags(deg', 0, n, n);
matI = speye(n);

%% Initialization: pick high degree node, and initialize as positive.
% Set all its neighbors in the opposite class.
phi = zeros(n,1);
[ ~, idx ] = max(deg);
neighbors = find(Asmall(idx,:));
phi(idx) = positive;
phi(neighbors) = negative;

%% FaBP: main equation
b = ( matI + a * D - c * Asmall ) \ phi;

%% Find the members of the two sets
set1 = b > 0;
set2 = b < 0;

% 1s in the error matrix
% missing edges in bc + extra edges within sets
E(1) = 2* (sum(set1)*sum(set2)-nnz(Asmall(set1,set2))) + nnz(Asmall(set1, set1)) + nnz(Asmall(set2, set2));
% 0s in the error matrix
E(2) = n^2 - E(1);

%% MDL cost of encoding given substructure as a bipartite core
MDLcostBC = richcomm_Cost( 'bc', N_tot, sum(set1), E, sum(set2) );


% extra edges within sets
E(1) = nnz(Asmall(set1, set1)) + nnz(Asmall(set2, set2));
% 0s in the error matrix
E(2) = n^2 - E(1);

%% MDL cost of encoding given substructure as a bipartite core
MDLcostNB = richcomm_Cost( 'nb', N_tot, sum(set1), E, sum(set2), [nnz(Asmall(set1,set2)), sum(set1)*sum(set2)-nnz(Asmall(set1,set2))] );

    
end

% Encode the connected component from SlashBurn.
function [ exact_found,strName,costGain,costGain_notEnc ] = richcomm_ExactCommStructure( Asmall, curind, N_tot)
minSize=3;strName='Not Sure';costGain=0;costGain_notEnc=0;

exact_found = false;
n = size(curind, 2);
m = nnz(Asmall);
if n==1
    return;
end

% cost of encoding the structure as near-clique
MDLcost_nc = richcomm_Cost( 'nc', N_tot, n, Asmall);
% cost of not encoding the structure at all (noise)
cost_notEnc = richcomm_Cost( 'err', 0, 0, [nnz(Asmall) n^2-nnz(Asmall)]);

if ( m == n*n)      % full clique
     
    if n ~= 2
        MDLcost_fc = richcomm_Cost( 'fc', N_tot, n, zeros(n,n));
        costGain = MDLcost_nc - MDLcost_fc;
        costGain_notEnc = cost_notEnc - MDLcost_fc;
       exact_found = true;strName='Full Clique';
     
    elseif n==2
        MDLcost_ch = richcomm_Cost( 'ch', N_tot, n, zeros(n,n));
        costGain = MDLcost_nc - MDLcost_ch;
        costGain_notEnc = cost_notEnc - MDLcost_ch;
        strName='Chain';

        exact_found = true;

    end
elseif (m == 2*(n-1))   % chain or star
    Asmall(Asmall>0)=1;
    degree = sum(Asmall);
    ind = find(degree > 0);
    d1count=0;
    d2count=0;
    dn1count=0;
    
    for i=1:size(degree, 2)
        if( degree(i) == 1 )
            d1count = d1count + 1;
        elseif degree(i) == 2
            d2count = d2count + 1;
        elseif degree(i) == n-1
            dn1count = dn1count + 1;
        end
    end
    if d1count == 2 && d2count == n-2     % chain
        MDLcost_ch = richcomm_Cost( 'ch', N_tot, n, zeros(n,n));
        costGain = MDLcost_nc - MDLcost_ch;costGain_notEnc = cost_notEnc - MDLcost_ch;
        exact_found = true;     strName='Chain';
    else
         strName='Chain or Star';
    end
    
else
    Asmall(Asmall>0)=1;
    opts.tol = 1e-2; 
    evals = eigs(Asmall, 2, 'lm', opts); % the eigenvalues with maximum magnitude
    [ set1, set2 ] = BFScoloring( Asmall );
    if ( max(evals) == - min(evals) )  % bipartite graph (special case: star)
        [ set1, set2 ] = BFScoloring( Asmall );
        if length(set1)+length(set2) < minSize
            exact_found = true;
            MDLcost_bc = richcomm_Cost( 'bc', N_tot, length(set1), zeros(n,n), length(set2));
            costGain = MDLcost_nc - MDLcost_bc;
            costGain_notEnc = cost_notEnc - MDLcost_bc;
            strName='Bipartite (special case: star)';
            return;
        end
        if length(set1) == 1 && length(set2) == 1
            MDLcost_ch = richcomm_Cost( 'ch', N_tot, n, zeros(n,n));
            costGain = MDLcost_nc - MDLcost_ch;
            costGain_notEnc = cost_notEnc - MDLcost_ch;
            strName='Chain';
            exact_found = true;
           elseif length(set1) == 1
            MDLcost_st = richcomm_Cost( 'st', N_tot, n, zeros(n,n));
            costGain = MDLcost_nc - MDLcost_st;
            costGain_notEnc = cost_notEnc - MDLcost_st;
            
            exact_found = true;
            strName='Star';
        elseif length(set2) == 1
            MDLcost_st = richcomm_Cost( 'st', N_tot, n, zeros(n,n));
            costGain = MDLcost_nc - MDLcost_st;
            costGain_notEnc = cost_notEnc - MDLcost_st;
            
            exact_found = true;
              strName='Star';
        else % bipartite graph
            degrees = sum(Asmall,2);
 
            if sum(full(degrees(set1)) ~= length(set2)*ones(length(set1),1)) && ...
                    sum(full(degrees(set2)) ~= length(set1)*ones(length(set2),1)) == 0
                MDLcost_bc = richcomm_Cost( 'bc', N_tot, length(set1), zeros(n,n), length(set2));
                costGain = MDLcost_nc - MDLcost_bc;
                costGain_notEnc = cost_notEnc - MDLcost_bc;
                
                exact_found = true;
                 strName='Bipartite';
            else
                MDLcost_bc = richcomm_Cost( 'bc', N_tot, length(set1), zeros(n,n), length(set2));
                MDLcost_nb = richcomm_Cost( 'nb', N_tot, length(set1), zeros(n,n), length(set2));
                if MDLcost_bc <= MDLcost_nb
                    costGain = MDLcost_nc - MDLcost_bc;
                    costGain_notEnc = cost_notEnc - MDLcost_bc;
                    exact_found = true;
                    strName='Bipartite';
                else % better to encode it as near-bipartite core
                    costGain = MDLcost_nc - MDLcost_nb;
                    costGain_notEnc = cost_notEnc - MDLcost_nb;
                    exact_found = true;
                    strName='Near Bipartite';
                end
            end
      end
    else
        
                    exact_found = false;
                    strName='Not Sure';costGain=0;costGain_notEnc=0;
        
    end
end
end

function [ set1, set2 ] = BFScoloring( Asmall )
%% Given a bipartite graph, find the two node-sets memberships

n = size(Asmall,2);
[seed ~] = find(Asmall);
queue = [min(seed)];
% sets = 2 (if unvisited) or 0 (if in set 1) or 1 (if in set 2)
sets = zeros(1,n)+2;
color = 1;
% coloring node 1 with color "0"
sets(min(seed)) = 0;
usedColor = false;


while ~( isempty(queue) )
    neighbors = find( Asmall(queue(1),:) ) ;
    
    for i = 1 : length(neighbors)
        if sets( neighbors(i) ) == 2 % unvisited neighbor
            sets( neighbors(i) ) = color;
            queue = [ queue, neighbors(i) ];
            usedColor = true;
        end
    end
    
    qsize = length(queue);
    queue = queue(2:qsize);
    if usedColor
        color = mod(color+1, 2);
    end
    usedColor = false;
end

set1 = find( sets == 0 );
set2 = find( sets == 1 );

end
function [ MDLcost ] = richcomm_Cost( subgraph, N_tot, n_sub, E, n_sub2, nb_edges)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

switch subgraph
    case 'fc'
        if E(1) == 0 || E(2) == 0 % no excluded edges
            MDLcost = LN( n_sub ) + l2cnk(N_tot, n_sub); %log2( nchoosek(vpi(N_tot), n_sub) );
        else
            MDLcost = LN( n_sub ) + l2cnk(N_tot, n_sub) + Lnu_opt(E);  %log2( nchoosek(vpi(N_tot), n_sub) ) + Lnu_opt( E );
        end
    case 'nc'
        % for the near clique: E is the Asmall matrix
        edges_inc = nnz(E);
        edges_exc = size(E,1)*size(E,2)-nnz(E);%sum(E(:)==0)-n_sub; % diagonal elements are always 0 (no self-loops)
        
        if edges_exc ~= 0 && edges_inc ~= 0
            MDLcost = LN( n_sub ) + l2cnk(N_tot, n_sub) + ... %log2(nchoosek(vpi(N_tot), n_sub)) + ...
                log2( n_sub^2 ) + edges_inc * NLL( edges_inc, edges_exc, 1) + ...
                edges_exc * NLL( edges_inc, edges_exc, 0);
        else
            MDLcost = LN( n_sub ) + l2cnk(N_tot, n_sub);
        end
    case 'st'
        if E(1) == 0 || E(2) == 0 %if sum(sum(E)) == 0
            MDLcost = LN( n_sub-1 ) + log2( N_tot ) +  l2cnk(N_tot-1, n_sub-1); %log2( nchoosek( vpi(N_tot-1), n_sub-1 ) );
        else
            MDLcost = LN( n_sub-1 ) + log2( N_tot ) + ...
                l2cnk(N_tot-1, n_sub-1) + ...  %log2( nchoosek( vpi(N_tot-1), n_sub-1 ) ) + ...
                Lnu_opt( E );
        end
    case 'ch'
        x = 0:(n_sub-1);
        N_tot_vec = N_tot * ones(1, n_sub);
        if E(1) == 0 || E(2) == 0 %if sum(sum(E)) == 0
            MDLcost = LN( n_sub-1 ) + sum( log2(N_tot_vec - x) );
        else
            MDLcost =  LN( n_sub-1 ) + sum( log2(N_tot_vec - x) ) + Lnu_opt( E );
        end
    case 'bc'
        k = n_sub;
        l = n_sub2;
        if E(1) == 0 || E(2) == 0 %if sum(sum(E)) == 0
            MDLcost = LN(k) + LN(l) + l2cnk(N_tot, k) + l2cnk(N_tot-k, l); %log2( nchoosek(vpi(N_tot), k) ) + log2( nchoosek(vpi(N_tot-k), l) );
        else
            MDLcost = LN(k) + LN(l) + l2cnk(N_tot, k) + ...            %log2( nchoosek(vpi(N_tot), k) ) + ...
                l2cnk(N_tot-k, l)        + ...      %log2( nchoosek(vpi(N_tot-k), l) ) + ...
                Lnu_opt( E );
        end
    case 'nb'
        k = n_sub;
        l = n_sub2;
        if E(1) == 0 || strcmp('ca', true) || E(2) == 0 %if sum(sum(E)) == 0
            MDLcost = LN(k) + LN(l) + l2cnk(N_tot, k) + l2cnk(N_tot-k, l);         % log2( nchoosek(vpi(N_tot), k) ) + log2( nchoosek(vpi(N_tot-k), l) );
        else
            % The error matrix E has only the edges *within* the two sets
            % of nodes.
            edges_inc = nb_edges(1); %nnz(nb_mat);
            edges_exc = nb_edges(2); %2*k*l - edges_inc; % the bipartite core model has 2*k*l edges (all the edges between the two sets of nodes - counting them 2 times, because the adjacency matrix is symmetric)
            if edges_inc == 0 || edges_exc == 0
                MDLcost = LN(k) + LN(l) + l2cnk(N_tot, k) + l2cnk(N_tot-k, l) +  Lnu_opt( E);  ... %log2( nchoosek(vpi(N_tot), k) ) + ...
            else
                MDLcost = LN(k) + LN(l) + l2cnk(N_tot, k) + l2cnk(N_tot-k, l) + ...         %log2( nchoosek(vpi(N_tot), k) ) + log2( nchoosek(vpi(N_tot-k), l) ) + 
                    log2( (n_sub+n_sub2)^2 ) + ...
                    edges_inc * NLL( edges_inc, edges_exc, 1) + ...
                    edges_exc * NLL( edges_inc, edges_exc, 0) + Lnu_opt( E );
                
            end
        end
    case 'err'
        if E(1) ~= 0 && E(2) ~= 0
            MDLcost = Lnu_opt( E );
        elseif E(1) ~= 0 
            MDLcost = LN( E(1) );
        elseif E(2) ~= 0
            MDLcost = LN( E(2) );
        end
    otherwise
        error_message = 'error: invalid structure...\n';
        disp(error_message);
end
end
%% encoded size of an integer >=1 as by Rissanen's 1983 Universal code for integers
    function [ c ] = LN( n )
        c0 = 2.865064;
        c = log2(c0);
        logTerm = abs(log2(n));
        while logTerm > 0
            c = c + logTerm;
            logTerm = log2(logTerm);
            if logTerm==Inf
                c=1;
                return
            end
        end
    end

%% error per structure: Naive Uniform
    function [ c_err ] = Lnu( E )
        Einc = nnz(E);
        Eexc = sum(E(:)==0);
        c_err = LN( Einc ) + ...
            Einc * NLL( Einc, Eexc, 1) + ...
            Eexc * NLL( Einc, Eexc, 0);
    end

%% error per full clique: Naive Uniform
    function [ c_err ] = Lnu_opt( E )
        % E has two entries: # of included edges, # of excluded edges
        Einc = E(1);
        Eexc = E(2);
        c_err = LN( Einc ) + ...
            Einc * NLL( Einc, Eexc, 1) + ...
            Eexc * NLL( Einc, Eexc, 0);
    end


%% Alternative error per structure: Naive Data-to-Model
    function [ c_err ] = Lnd( E )
        Einc = nnz(E);
        c_err = LN( Einc ) + l2cnk (N_tot^2, Einc);   %log2( nchoosek(vpi(N_tot^2), Einc) );
    end


%% Alternative error per structure: Naive Data-to-Model
    function [ c_err ] = Lnd_opt( E )
        Einc = E(1);
        c_err = LN( Einc ) + l2cnk (N_tot^2, Einc);  %log2( nchoosek(vpi(N_tot^2), Einc) );
    end

%% Negative log-likelihood
% If sub = 0: p0 = -log2(excl / (incl + excl))
% if sub = 1: p1 = -log2(incl / (incl + excl))
    function [ l ] = NLL( incl, excl, sub )
        if sub == 0
            l = -log2(excl / (incl + excl));
        elseif sub == 1
            l = -log2(incl / (incl + excl));
        else
            err = 'error... Can only compute l0 ot l1 (negative log-likelihood)'
        end
    end
function [nbits] = l2cnk(n,k)
	nbits = 0;
	for i = n:-1:n-k+1
		nbits = nbits + log2(i);
	end
	
	for i = k:-1:1
		nbits = nbits - log2(i);
	end
end

 function newmodel=getuniqueVals(newmodel)
    currentlength=length(newmodel);
    row=1;
    while row<currentlength 
        nodes1=newmodel(row).Nodes;
        newmodel=getuniqueVals2(newmodel,nodes1,row);
        currentlength=length(newmodel);
        row=row+1;
    end
end
function model=getuniqueVals2(model,nodes1,j)
    k=j+1;currentlength=length(model);
    while k<currentlength
             nodes2=model(k).Nodes;
             l=isequal(nodes1,nodes2);
             %m=length(setdiff(nodes1,nodes2));
             if(l==1)
                 model(k)=[];
                 currentlength=length(model);
             else
                 k=k+1;
                 currentlength=length(model);
             end
    end
end